import { ASTPosition } from 'miniscript-core';
import { CustomValue } from '../types/base';
import { CustomFunctionCallback } from '../types/function';
import { CustomString } from '../types/string';
export declare enum OpCode {
    NOOP = 0,
    HALT = 1,
    CALL = 2,
    CALL_INTERNAL = 3,
    CALL_WITH_CONTEXT = 4,
    CONSTRUCT_MAP = 5,
    CONSTRUCT_LIST = 6,
    GET_VARIABLE = 7,
    GET_PROPERTY = 8,
    GET_ENVAR = 9,
    GET_SELF = 10,
    GET_OUTER = 11,
    GET_GLOBALS = 12,
    GET_LOCALS = 13,
    GET_SUPER = 14,
    GET_SUPER_PROPERTY = 15,
    CALL_SUPER_PROPERTY = 16,
    FALSIFY = 17,
    NEGATE = 18,
    NEW = 19,
    SLICE = 20,
    ASSIGN = 21,
    PUSH = 22,
    POP = 23,
    ISA = 24,
    ADD = 25,
    SUB = 26,
    MUL = 27,
    DIV = 28,
    MOD = 29,
    POW = 30,
    EQUAL = 31,
    NOT_EQUAL = 32,
    LESS_THAN = 33,
    LESS_THAN_OR_EQUAL = 34,
    GREATER_THAN = 35,
    GREATER_THAN_OR_EQUAL = 36,
    AND = 37,
    OR = 38,
    RETURN = 39,
    FUNCTION_DEFINITION = 40,
    GOTO_A = 41,
    GOTO_A_IF_FALSE = 42,
    GOTO_A_IF_FALSE_AND_PUSH = 43,
    GOTO_A_IF_TRUE_AND_PUSH = 44,
    PUSH_ITERATOR = 45,
    POP_ITERATOR = 46,
    NEXT = 47,
    BITWISE_OR = 48,
    BITWISE_AND = 49,
    BITWISE_LEFT_SHIFT = 50,
    BITWISE_RIGHT_SHIFT = 51,
    BITWISE_UNSIGNED_RIGHT_SHIFT = 52,
    BREAKPOINT = 53,
    BREAKPOINT_ENABLE = 54,
    IMPORT = 55
}
export interface FunctionDefinitionInstructionArgument {
    name: CustomString;
    defaultValue: CustomValue;
}
export type SourceLocation = {
    name: string;
    path: string;
    start: ASTPosition;
    end: ASTPosition;
};
export interface BaseInstruction {
    op: OpCode;
    source: SourceLocation;
    ip?: number;
}
export interface GetVariableInstruction extends BaseInstruction {
    op: OpCode.GET_VARIABLE;
    property: CustomString;
    invoke: boolean;
}
export interface GetPropertyInstruction extends BaseInstruction {
    op: OpCode.GET_PROPERTY | OpCode.GET_SUPER_PROPERTY;
    invoke: boolean;
}
export interface PushInstruction extends BaseInstruction {
    op: OpCode.PUSH;
    value: CustomValue;
}
export interface ConstructMapInstruction extends BaseInstruction {
    op: OpCode.CONSTRUCT_MAP;
    length: number;
}
export interface ConstructListInstruction extends BaseInstruction {
    op: OpCode.CONSTRUCT_LIST;
    length: number;
}
export interface CallInstruction extends BaseInstruction {
    op: OpCode.CALL;
    length: number;
}
export interface CallWithContextInstruction extends BaseInstruction {
    op: OpCode.CALL_WITH_CONTEXT | OpCode.CALL_SUPER_PROPERTY;
    length: number;
}
export interface FunctionDefinitionInstruction extends BaseInstruction {
    op: OpCode.FUNCTION_DEFINITION;
    arguments: FunctionDefinitionInstructionArgument[];
    code: Instruction[];
    ignoreOuter: boolean;
}
export interface GotoAInstruction extends BaseInstruction {
    op: OpCode.GOTO_A | OpCode.GOTO_A_IF_FALSE | OpCode.GOTO_A_IF_FALSE_AND_PUSH | OpCode.GOTO_A_IF_TRUE_AND_PUSH;
    goto: Instruction;
}
export interface NextInstruction extends BaseInstruction {
    op: OpCode.NEXT;
    goto: Instruction;
    idxVariable: CustomString;
    variable: CustomString;
}
export interface CallInternalInstruction extends BaseInstruction {
    op: OpCode.CALL_INTERNAL;
    callback: CustomFunctionCallback;
    arguments: FunctionDefinitionInstructionArgument[];
}
export interface ImportInstruction extends BaseInstruction {
    op: OpCode.IMPORT;
    path: string;
}
export type Instruction = BaseInstruction | GetVariableInstruction | PushInstruction | ConstructMapInstruction | ConstructListInstruction | FunctionDefinitionInstruction | GotoAInstruction | GetPropertyInstruction | CallInstruction | CallWithContextInstruction | NextInstruction | CallInternalInstruction | ImportInstruction;
