import { ASTFeatureEnvarExpression, ASTFeatureImportExpression, ASTFeatureIncludeExpression } from 'greybel-core';
import { ASTAssignmentStatement, ASTBase, ASTCallExpression, ASTEvaluationExpression, ASTForGenericStatement, ASTFunctionStatement, ASTIdentifier, ASTIfStatement, ASTListConstructorExpression, ASTLiteral, ASTMapConstructorExpression, ASTReturnStatement, ASTUnaryExpression, ASTWhileStatement, ASTSliceExpression, ASTIndexExpression, ASTMemberExpression } from 'miniscript-core';
import { Instruction, SourceLocation } from './byte-compiler/instruction';
import { Stack } from './utils/stack';
import { HandlerContainer } from './handler-container';
export interface LineContext {
    isCommand?: boolean;
    includeOuter?: boolean;
}
export interface LineCallableContext extends LineContext {
    isReference?: boolean;
}
export interface LineIdentifierContext extends LineCallableContext {
    isDescending?: boolean;
}
export interface BytecodeCompileResult {
    code: Instruction[];
    imports: Map<string, Instruction[]>;
}
export interface BytecodeGeneratorContext {
    code: Instruction[];
    jumpPoints: [Instruction, Instruction][];
}
export interface BytecodeConverterOptions {
    target: string;
    handler: HandlerContainer;
    debugMode?: boolean;
}
export declare class BytecodeGenerator {
    protected handler: HandlerContainer;
    protected context: Stack<BytecodeGeneratorContext>;
    protected target: Stack<string>;
    protected debugMode: boolean;
    protected imports: Map<string, Instruction[]>;
    constructor(options: BytecodeConverterOptions);
    parse(code: string): ASTBase | import("greybel-core").ASTChunkAdvanced;
    compile(code: string): Promise<BytecodeCompileResult>;
    protected getCurrentPointer(): number;
    protected getSourceLocation(node: ASTBase, name?: string): SourceLocation;
    protected getInternalLocation(): SourceLocation;
    protected pushContext(): void;
    protected popContext(): BytecodeGeneratorContext;
    protected getLastJumpPoint(): [Instruction, Instruction];
    protected pushJumppoint(start: Instruction, end: Instruction): void;
    protected popJumppoint(): [Instruction, Instruction];
    protected push(item: Instruction): void;
    protected processNode(node: ASTBase): Promise<void>;
    protected processSubNode(node: ASTBase, context?: LineContext): Promise<void>;
    protected processMemberExpression(node: ASTMemberExpression, context?: LineCallableContext): Promise<void>;
    protected processIndexExpression(node: ASTIndexExpression, context?: LineCallableContext): Promise<void>;
    protected processSliceExpression(node: ASTSliceExpression): Promise<void>;
    protected processIdentifier(node: ASTIdentifier, context?: LineIdentifierContext): Promise<void>;
    protected processAssignmentStatement(node: ASTAssignmentStatement): Promise<void>;
    protected processLiteral(node: ASTLiteral): Promise<void>;
    protected processEvaluationExpression(node: ASTEvaluationExpression): Promise<void>;
    protected processReturn(node: ASTReturnStatement): Promise<void>;
    protected processBreak(node: ASTBase): Promise<void>;
    protected processContinue(node: ASTBase): Promise<void>;
    protected processMapConstructorExpression(node: ASTMapConstructorExpression): Promise<void>;
    protected processListConstructorExpression(node: ASTListConstructorExpression): Promise<void>;
    protected processFunctionDeclaration(node: ASTFunctionStatement, context?: LineContext): Promise<void>;
    protected processWhileStatement(node: ASTWhileStatement): Promise<void>;
    protected processUnaryExpression(node: ASTUnaryExpression): Promise<void>;
    protected processCallExpression(node: ASTCallExpression): Promise<void>;
    protected processIfStatement(node: ASTIfStatement): Promise<void>;
    protected processForGenericStatement(node: ASTForGenericStatement): Promise<void>;
    protected processEnvarExpression(node: ASTFeatureEnvarExpression): Promise<void>;
    protected createImport(node: ASTFeatureImportExpression, path: string, code: string): Promise<void>;
    protected processImportExpression(node: ASTFeatureImportExpression): Promise<void>;
    protected processIncludeExpression(node: ASTFeatureIncludeExpression): Promise<void>;
    protected processDebuggerExpression(node: ASTBase): Promise<void>;
}
