import { CustomValue } from './types/base';
import { CustomMap } from './types/map';
import { ObjectValue } from './utils/object-value';
import { ContextTypeIntrinsics } from './context/types';
import { Instruction } from './byte-compiler/instruction';
import { CustomValueWithIntrinsicsResult } from './types/with-intrinsics';
import { Stack } from './utils/stack';
export declare enum ContextType {
    Api = 0,
    Global = 1,
    Function = 2,
    Injected = 3
}
export declare class Scope extends CustomMap {
    private readonly context;
    constructor(context: OperationContext);
    get(current: CustomValue, typeIntrinsics: ContextTypeIntrinsics): CustomValue;
    getWithOrigin(current: CustomValue, typeIntrinsics: ContextTypeIntrinsics): CustomValueWithIntrinsicsResult;
}
export interface ContextOptions {
    code: Instruction[];
    previous?: OperationContext;
    type?: ContextType;
    self?: CustomValue;
    super?: CustomValue;
    isProtected?: boolean;
    outer?: OperationContext;
}
export interface ContextForkOptions {
    code: Instruction[];
    type: ContextType;
    self?: CustomValue;
    super?: CustomValue;
    target?: string;
    outer?: OperationContext;
}
export declare class OperationContext {
    previous: OperationContext;
    iterators: Stack<Iterator<CustomValue> & {
        index: number;
    }>;
    readonly type: ContextType;
    readonly scope: Scope;
    isProtected: boolean;
    injected: boolean;
    cp: null | number;
    ip: number;
    code: Instruction[];
    self: CustomValue | null;
    super: CustomValue | null;
    readonly api: OperationContext;
    readonly locals: OperationContext;
    readonly outer: OperationContext;
    readonly globals: OperationContext;
    private static readonly lookupApiType;
    private static readonly lookupGlobalsType;
    private static readonly lookupLocalsType;
    constructor(options: ContextOptions);
    lookupAllOfType(validate: (type: ContextType) => boolean): OperationContext[];
    lookupType(allowedTypes: Array<ContextType>): OperationContext;
    lookupAllScopes(): OperationContext[];
    lookupApi(): OperationContext;
    lookupGlobals(): OperationContext;
    lookupLocals(): OperationContext;
    lookupOuter(): OperationContext;
    extend(map: ObjectValue): OperationContext;
    set(path: CustomValue, value: CustomValue): void;
    get(path: CustomValue, contextTypeIntrinsics: ContextTypeIntrinsics): CustomValue;
    injectContext(): void;
    getCurrentInstruction(): Instruction | null;
    fork(options: ContextForkOptions): OperationContext;
}
