import type { VM } from '../vm';
export interface KeyEvent {
    keyCode?: number;
    charCode?: number;
    code: string;
}
export interface PrintOptions {
    appendNewLine: boolean;
    replace: boolean;
}
export interface UpdateOptions {
    appendNewLine: boolean;
    replace: boolean;
}
export declare abstract class OutputHandler {
    abstract print(vm: VM, message: string, options?: Partial<PrintOptions>): void;
    abstract update(vm: VM, message: string, options?: Partial<UpdateOptions>): void;
    abstract progress(vm: VM, timeout: number): PromiseLike<void>;
    abstract waitForInput(vm: VM, isPassword: boolean, message?: string): PromiseLike<string>;
    abstract waitForKeyPress(vm: VM, message?: string): PromiseLike<KeyEvent>;
    abstract clear(vm: VM): void;
}
export declare class DefaultOutputHandler extends OutputHandler {
    print(_vm: VM, message: string, { appendNewLine }?: Partial<PrintOptions>): void;
    update(_vm: VM, message: string, { appendNewLine, replace }?: Partial<UpdateOptions>): void;
    progress(_vm: VM, timeout: number): PromiseLike<void>;
    waitForInput(_vm: VM, _isPassword: boolean, _message?: string): PromiseLike<string>;
    waitForKeyPress(_vm: VM, _message?: string): PromiseLike<KeyEvent>;
    clear(_vm: VM): void;
}
