/// <reference types="node" />
import { EventEmitter } from 'events';
import { OperationContext } from './context';
import { HandlerContainer } from './handler-container';
import { CustomValue } from './types/base';
import { CustomString } from './types/string';
import { ObjectValue } from './utils/object-value';
import { Debugger, VM, VMOptions } from './vm';
import { BytecodeCompileResult } from './bytecode-generator';
export declare const PARAMS_PROPERTY: CustomString;
export declare const IS_GREYBEL_PROPERTY: CustomString;
export interface InterpreterOptions {
    target?: string;
    api?: ObjectValue;
    params?: Array<string>;
    handler?: HandlerContainer;
    debugger?: Debugger;
    debugMode?: boolean;
    environmentVariables?: Map<string, string>;
}
export interface InterpreterRunOptions {
    customCode?: string;
    vmOptions?: Partial<VMOptions>;
}
export declare class Interpreter extends EventEmitter {
    target: string;
    api: ObjectValue;
    params: Array<string>;
    environmentVariables: Map<string, string>;
    handler: HandlerContainer;
    debugger: Debugger;
    debugMode: boolean;
    apiContext: OperationContext;
    globalContext: OperationContext;
    vm: VM;
    constructor(options?: InterpreterOptions);
    setTarget(target: string): Interpreter;
    setDebugger(dbgr: Debugger): Interpreter;
    setApi(newApi: ObjectValue): Interpreter;
    setHandler(handler: HandlerContainer): Interpreter;
    inject(code: string, context?: OperationContext): Promise<Interpreter>;
    injectInLastContext(code: string): Promise<Interpreter>;
    protected initVM(result: BytecodeCompileResult, options?: Partial<VMOptions>): void;
    protected start(): Promise<Interpreter>;
    run({ customCode, vmOptions }?: InterpreterRunOptions): Promise<Interpreter>;
    resume(): Interpreter;
    pause(): Interpreter;
    exit(): void;
    setGlobalVariable(path: string, value: CustomValue): Interpreter;
    getGlobalVariable(path: string): CustomValue;
}
