"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Interpreter = exports.IS_GREYBEL_PROPERTY = exports.PARAMS_PROPERTY = void 0;
const events_1 = require("events");
const context_1 = require("./context");
const handler_container_1 = require("./handler-container");
const default_1 = require("./types/default");
const function_1 = require("./types/function");
const list_1 = require("./types/list");
const map_1 = require("./types/map");
const number_1 = require("./types/number");
const string_1 = require("./types/string");
const error_1 = require("./utils/error");
const object_value_1 = require("./utils/object-value");
const boolean_1 = require("./types/boolean");
const vm_1 = require("./vm");
const bytecode_generator_1 = require("./bytecode-generator");
exports.PARAMS_PROPERTY = new string_1.CustomString('params');
exports.IS_GREYBEL_PROPERTY = new string_1.CustomString('IS_GREYBEL');
class Interpreter extends events_1.EventEmitter {
    constructor(options = {}) {
        var _a, _b, _c, _d, _e, _f, _g;
        super();
        this.handler = (_a = options.handler) !== null && _a !== void 0 ? _a : new handler_container_1.HandlerContainer();
        this.debugger = (_b = options.debugger) !== null && _b !== void 0 ? _b : new vm_1.Debugger();
        this.vm = null;
        this.api = (_c = options.api) !== null && _c !== void 0 ? _c : new object_value_1.ObjectValue();
        this.params = (_d = options.params) !== null && _d !== void 0 ? _d : [];
        this.environmentVariables = (_e = options.environmentVariables) !== null && _e !== void 0 ? _e : new Map();
        this.debugMode = (_f = options.debugMode) !== null && _f !== void 0 ? _f : false;
        this.apiContext = null;
        this.globalContext = null;
        this.setTarget((_g = options.target) !== null && _g !== void 0 ? _g : 'unknown');
    }
    setTarget(target) {
        if (this.vm !== null && this.vm.isPending()) {
            throw new Error('You cannot set a target while a process is running.');
        }
        this.target = target;
        return this;
    }
    setDebugger(dbgr) {
        if (this.vm !== null && this.vm.isPending()) {
            throw new Error('You cannot set a debugger while a process is running.');
        }
        this.debugger = dbgr;
        return this;
    }
    setApi(newApi) {
        if (this.vm !== null && this.vm.isPending()) {
            throw new Error('You cannot set a api while a process is running.');
        }
        this.api = newApi;
        return this;
    }
    setHandler(handler) {
        if (this.vm !== null && this.vm.isPending()) {
            throw new Error('You cannot set a handler while a process is running.');
        }
        this.handler = handler;
        return this;
    }
    inject(code, context) {
        return __awaiter(this, void 0, void 0, function* () {
            const bytecodeGenerator = new bytecode_generator_1.BytecodeGenerator({
                target: 'injected',
                handler: this.handler
            });
            const result = yield bytecodeGenerator.compile(code);
            const vm = new vm_1.VM({
                target: this.target,
                debugger: this.debugger,
                handler: this.handler,
                environmentVariables: this.environmentVariables,
                contextTypeIntrinsics: this.vm.contextTypeIntrinsics,
                globals: (context !== null && context !== void 0 ? context : this.globalContext).fork({
                    code: result.code,
                    type: context_1.ContextType.Injected
                }),
                imports: result.imports
            });
            try {
                yield vm.exec();
            }
            catch (err) {
                if (err instanceof error_1.PrepareError || err instanceof error_1.RuntimeError) {
                    this.handler.errorHandler.raise(err);
                }
                else {
                    this.handler.errorHandler.raise(new error_1.RuntimeError(err.message, vm, err));
                }
            }
            return this;
        });
    }
    injectInLastContext(code) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.vm !== null && this.vm.isPending()) {
                const last = this.vm.getFrame();
                return this.inject(code, last);
            }
            throw new Error('Unable to inject into last context.');
        });
    }
    initVM(result, options) {
        const apiContext = new context_1.OperationContext({
            isProtected: true,
            code: [],
        });
        const globalContext = apiContext.fork({
            type: context_1.ContextType.Global,
            code: result.code
        });
        const vm = new vm_1.VM(Object.assign({ target: this.target, debugger: this.debugger, handler: this.handler, environmentVariables: this.environmentVariables, contextTypeIntrinsics: {
                string: string_1.CustomString.getIntrinsics().fork(),
                number: number_1.CustomNumber.getIntrinsics().fork(),
                list: list_1.CustomList.getIntrinsics().fork(),
                map: map_1.CustomMap.getIntrinsics().fork(),
                function: function_1.CustomFunction.intrinsics.fork()
            }, globals: globalContext, imports: result.imports }, options));
        const stringIntrinsics = map_1.CustomMap.createWithInitialValue(vm.contextTypeIntrinsics.string);
        const numberIntrinsics = map_1.CustomMap.createWithInitialValue(vm.contextTypeIntrinsics.number);
        const listIntrinsics = map_1.CustomMap.createWithInitialValue(vm.contextTypeIntrinsics.list);
        const mapIntrinsics = map_1.CustomMap.createWithInitialValue(vm.contextTypeIntrinsics.map);
        const funcRefIntrinsics = map_1.CustomMap.createWithInitialValue(vm.contextTypeIntrinsics.function);
        apiContext.scope.set(new string_1.CustomString('string'), stringIntrinsics);
        apiContext.scope.set(new string_1.CustomString('number'), numberIntrinsics);
        apiContext.scope.set(new string_1.CustomString('list'), listIntrinsics);
        apiContext.scope.set(new string_1.CustomString('map'), mapIntrinsics);
        apiContext.scope.set(new string_1.CustomString('funcRef'), funcRefIntrinsics);
        apiContext.scope.extend(this.api);
        const newParams = new list_1.CustomList(this.params.map((item) => new string_1.CustomString(item)));
        globalContext.scope.set(exports.IS_GREYBEL_PROPERTY, new boolean_1.CustomBoolean(true));
        globalContext.scope.set(exports.PARAMS_PROPERTY, newParams);
        this.vm = vm;
        this.apiContext = apiContext;
        this.globalContext = globalContext;
    }
    start() {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                this.emit('start', this);
                yield this.vm.exec();
            }
            catch (err) {
                if (err instanceof error_1.PrepareError || err instanceof error_1.RuntimeError) {
                    this.handler.errorHandler.raise(err);
                }
                else {
                    this.handler.errorHandler.raise(new error_1.RuntimeError(err.message, this.vm, err));
                }
            }
            finally {
                this.emit('exit', this);
            }
            return this;
        });
    }
    run({ customCode, vmOptions } = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            const code = customCode !== null && customCode !== void 0 ? customCode : (yield this.handler.resourceHandler.get(this.target));
            const bytecodeConverter = new bytecode_generator_1.BytecodeGenerator({
                target: this.target,
                handler: this.handler,
                debugMode: this.debugMode
            });
            const bytecode = yield bytecodeConverter.compile(code);
            this.initVM(bytecode, vmOptions);
            return this.start();
        });
    }
    resume() {
        if (this.vm !== null && this.vm.isPending()) {
            this.debugger.setBreakpoint(false);
        }
        return this;
    }
    pause() {
        if (this.vm !== null && this.vm.isPending()) {
            this.debugger.setBreakpoint(true);
        }
        return this;
    }
    exit() {
        this.vm.exit();
    }
    setGlobalVariable(path, value) {
        if (this.globalContext != null) {
            this.globalContext.set(new string_1.CustomString(path), value);
        }
        return this;
    }
    getGlobalVariable(path) {
        if (this.globalContext != null) {
            return this.globalContext.get(new string_1.CustomString(path), this.vm.contextTypeIntrinsics);
        }
        return default_1.DefaultType.Void;
    }
}
exports.Interpreter = Interpreter;
