import { FunctionDefinitionInstructionArgument, Instruction } from '../byte-compiler/instruction';
import { OperationContext } from '../context';
import { ContextTypeIntrinsics } from '../context/types';
import { ObjectValue } from '../utils/object-value';
import type { VM } from '../vm';
import { CustomValue } from './base';
export interface CustomFunctionCallback {
    (vm: VM, self: CustomValue, args: Map<string, CustomValue>): Promise<NonNullable<CustomValue>>;
}
export declare class CustomFunction extends CustomValue {
    static readonly intrinsics: ObjectValue;
    static createExternal(name: string, callback: CustomFunctionCallback): CustomFunction;
    static createExternalWithSelf(name: string, callback: CustomFunctionCallback): CustomFunction;
    readonly outer?: OperationContext;
    readonly name: string;
    readonly id: string;
    readonly arguments: FunctionDefinitionInstructionArgument[];
    readonly value: Instruction[];
    constructor(name: string, value: Instruction[], args?: FunctionDefinitionInstructionArgument[], outer?: OperationContext);
    addArgument(name: string, defaultValue?: CustomValue): CustomFunction;
    fork(): CustomFunction;
    forkAs(name: string): CustomFunction;
    getCustomType(): string;
    toNumber(): number;
    toInt(): number;
    toJSON(): string;
    toString(): string;
    toTruthy(): boolean;
    instanceOf(v: CustomValue, typeIntrinsics: ContextTypeIntrinsics): boolean;
    hash(): number;
}
