"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CustomFunction = void 0;
const instruction_1 = require("../byte-compiler/instruction");
const keywords_1 = require("../byte-compiler/keywords");
const hash_1 = require("../utils/hash");
const object_value_1 = require("../utils/object-value");
const uuid_1 = require("../utils/uuid");
const base_1 = require("./base");
const default_1 = require("./default");
const nil_1 = require("./nil");
const string_1 = require("./string");
class CustomFunction extends base_1.CustomValue {
    static createExternal(name, callback) {
        const args = [];
        return new CustomFunction(name, [
            {
                op: instruction_1.OpCode.CALL_INTERNAL,
                source: {
                    name: 'internal',
                    path: 'internal',
                    start: { line: 0, character: 0 },
                    end: { line: 0, character: 0 }
                },
                callback,
                arguments: args,
                ip: 0
            },
            {
                op: instruction_1.OpCode.RETURN,
                source: {
                    name: 'internal',
                    path: 'internal',
                    start: { line: 0, character: 0 },
                    end: { line: 0, character: 0 }
                },
                ip: 1
            }
        ], args);
    }
    static createExternalWithSelf(name, callback) {
        return this.createExternal(name, callback).addArgument(keywords_1.RuntimeKeyword.Self);
    }
    constructor(name, value, args = [], outer) {
        super();
        this.id = (0, uuid_1.uuid)();
        this.name = name;
        this.value = value;
        this.arguments = args;
        this.outer = outer;
    }
    addArgument(name, defaultValue = default_1.DefaultType.Void) {
        this.arguments.push({
            name: new string_1.CustomString(name),
            defaultValue
        });
        return this;
    }
    fork() {
        return new CustomFunction(this.name, this.value, this.arguments, this.outer);
    }
    forkAs(name) {
        return new CustomFunction(name, this.value, this.arguments, this.outer);
    }
    getCustomType() {
        return 'function';
    }
    toNumber() {
        return Number.NaN;
    }
    toInt() {
        return 0;
    }
    toJSON() {
        return this.toString();
    }
    toString() {
        const args = this.arguments.map((item) => {
            const value = item.defaultValue;
            if (value instanceof nil_1.CustomNil) {
                return item.name;
            }
            else if (value instanceof string_1.CustomString) {
                return `${item.name}="${value.value}"`;
            }
            return `${item.name}=${value.value}`;
        });
        return `FUNCTION(${args.join(', ')})`;
    }
    toTruthy() {
        return true;
    }
    instanceOf(v, typeIntrinsics) {
        var _a;
        return v.value === ((_a = typeIntrinsics.function) !== null && _a !== void 0 ? _a : CustomFunction.intrinsics);
    }
    hash() {
        return (0, hash_1.getStringHashCode)(this.toString());
    }
}
CustomFunction.intrinsics = new object_value_1.ObjectValue();
exports.CustomFunction = CustomFunction;
