"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CustomList = exports.CustomListIterator = void 0;
const hash_1 = require("../utils/hash");
const object_value_1 = require("../utils/object-value");
const uuid_1 = require("../utils/uuid");
const number_1 = require("./number");
const with_intrinsics_1 = require("./with-intrinsics");
class CustomListIterator {
    constructor(value) {
        const me = this;
        me.value = value;
        me.index = 0;
    }
    next() {
        const me = this;
        if (me.index >= me.value.length) {
            return {
                value: null,
                done: true
            };
        }
        return {
            value: me.value[me.index++],
            done: false
        };
    }
}
exports.CustomListIterator = CustomListIterator;
class CustomList extends with_intrinsics_1.CustomObject {
    static getItemIndex(item, index) {
        let n = index | 0;
        if (n < 0)
            n += item.value.length;
        if (n < 0 || n >= item.value.length)
            return -1;
        return n;
    }
    constructor(value = []) {
        super();
        this.id = (0, uuid_1.uuid)();
        this.value = [...value];
    }
    getCustomType() {
        return 'list';
    }
    toJSON(depth = 0) {
        return this.toString(depth);
    }
    toString(depth = 0) {
        return `[${this.value.map((item) => item.toJSON(depth)).join(', ')}]`;
    }
    fork() {
        return new CustomList(this.value);
    }
    toNumber() {
        return 0;
    }
    toInt() {
        return 0;
    }
    toTruthy() {
        return this.value.length > 0;
    }
    getSize() {
        return this.value.length;
    }
    instanceOf(v, typeIntrinsics) {
        var _a;
        return v.value === ((_a = typeIntrinsics.list) !== null && _a !== void 0 ? _a : CustomList.intrinsics);
    }
    slice(a, b) {
        return new CustomList(this.value.slice(a.toNumber(), b.toNumber()));
    }
    extend(list) {
        if (list instanceof CustomList) {
            list = list.value;
        }
        for (let index = 0; index < list.length; index++) {
            this.value.push(list[index]);
        }
        return this;
    }
    [Symbol.iterator]() {
        return new CustomListIterator(this.value);
    }
    getItemIndex(index) {
        return CustomList.getItemIndex(this, index);
    }
    has(current) {
        if (current instanceof number_1.CustomNumber) {
            const currentIndex = this.getItemIndex(current.toInt());
            const sub = this.value[currentIndex];
            if (sub) {
                return true;
            }
        }
        return false;
    }
    set(current, newValue) {
        if (current instanceof number_1.CustomNumber) {
            const lastIndex = this.getItemIndex(current.toInt());
            if (lastIndex >= 0 && lastIndex < this.value.length) {
                this.value[lastIndex] = newValue;
                return;
            }
            throw new Error(`Index error (list index ${lastIndex} out of range).`);
        }
        throw new Error(`Index is not a number.`);
    }
    get(current, typeIntrinsics) {
        var _a;
        if (current instanceof number_1.CustomNumber) {
            const currentIndex = this.getItemIndex(current.toInt());
            if (currentIndex >= 0 && currentIndex < this.value.length) {
                return this.value[currentIndex];
            }
            throw new Error(`Index error (list index ${currentIndex} out of range).`);
        }
        const intrinsics = (_a = typeIntrinsics.list) !== null && _a !== void 0 ? _a : CustomList.getIntrinsics();
        if (intrinsics.has(current)) {
            return intrinsics.get(current);
        }
        throw new Error(`Unknown path in list ${current.toString()}.`);
    }
    getWithOrigin(current, typeIntrinsics) {
        return {
            value: this.get(current, typeIntrinsics),
            origin: null
        };
    }
    hash(recursionDepth = 0) {
        let result = (0, hash_1.getHashCode)(this.value.length);
        if (recursionDepth > 4)
            return result;
        this.value.forEach((value) => {
            result = (0, hash_1.rotateBits)(result) ^ value.hash(recursionDepth + 1);
        });
        return result;
    }
}
CustomList.intrinsics = new object_value_1.ObjectValue();
exports.CustomList = CustomList;
