"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CustomMap = exports.CustomMapIterator = exports.CUSTOM_MAP_MAX_DEPTH_VALUE = exports.CUSTOM_MAP_MAX_DEPTH = void 0;
const hash_1 = require("../utils/hash");
const object_value_1 = require("../utils/object-value");
const uuid_1 = require("../utils/uuid");
const nil_1 = require("./nil");
const string_1 = require("./string");
const with_intrinsics_1 = require("./with-intrinsics");
exports.CUSTOM_MAP_MAX_DEPTH = 2;
exports.CUSTOM_MAP_MAX_DEPTH_VALUE = '{...}';
class CustomMapIterator {
    constructor(value) {
        const me = this;
        me.value = value;
        me.index = 0;
    }
    next() {
        const me = this;
        const keys = Array.from(me.value.keys());
        if (me.index >= keys.length) {
            return {
                value: nil_1.Void,
                done: true
            };
        }
        const key = keys[me.index++];
        return {
            value: new CustomMap(new object_value_1.ObjectValue([
                [new string_1.CustomString('key'), key],
                [new string_1.CustomString('value'), me.value.get(key)]
            ])),
            done: false
        };
    }
}
exports.CustomMapIterator = CustomMapIterator;
class CustomMap extends with_intrinsics_1.CustomObject {
    static createWithInitialValue(value) {
        const map = new CustomMap();
        map.value = value;
        return map;
    }
    constructor(value) {
        super();
        this.isInstance = false;
        this.id = (0, uuid_1.uuid)();
        this.value = new object_value_1.ObjectValue(value);
    }
    getCustomType() {
        return 'map';
    }
    toJSON(depth = 0) {
        return this.toString(depth);
    }
    toString(depth = 0) {
        const fields = [];
        if (exports.CUSTOM_MAP_MAX_DEPTH < depth) {
            return exports.CUSTOM_MAP_MAX_DEPTH_VALUE;
        }
        for (const [key, value] of this.value.entries()) {
            fields.push(`${key instanceof CustomMap ? key.toJSON(depth + 1) : key.toJSON(depth)}:${value instanceof CustomMap
                ? value.toJSON(depth + 1)
                : value.toJSON(depth)}`);
        }
        return `{${fields.join(',')}}`;
    }
    fork() {
        return new CustomMap(this.value);
    }
    toNumber() {
        return 0;
    }
    toInt() {
        return 0;
    }
    toTruthy() {
        return this.value.size > 0;
    }
    getSize() {
        return this.value.size;
    }
    instanceOf(v, typeIntrinsics) {
        var _a;
        if (v instanceof CustomMap) {
            let current = this;
            while ((current = current.getIsa())) {
                if (current === v) {
                    return true;
                }
            }
            return v.value === ((_a = typeIntrinsics.map) !== null && _a !== void 0 ? _a : CustomMap.getIntrinsics());
        }
        return false;
    }
    [Symbol.iterator]() {
        return new CustomMapIterator(this.value);
    }
    extend(map) {
        if (map instanceof CustomMap) {
            map = map.value;
        }
        this.value.extend(map);
        return this;
    }
    has(current) {
        if (current !== null) {
            if (this.value.has(current)) {
                return true;
            }
            const isa = this.getIsa();
            return isa ? isa.has(current) : false;
        }
        return false;
    }
    set(current, newValue) {
        this.value.set(current, newValue);
    }
    get(current, typeIntrinsics) {
        var _a;
        const isa = this.getIsa();
        if (current !== null) {
            if (this.value.has(current)) {
                return this.value.get(current);
            }
            else if (isa === null || isa === void 0 ? void 0 : isa.has(current)) {
                return isa.get(current, typeIntrinsics);
            }
            const intrinsics = (_a = typeIntrinsics.map) !== null && _a !== void 0 ? _a : CustomMap.getIntrinsics();
            if (intrinsics.has(current)) {
                return intrinsics.get(current);
            }
        }
        throw new Error(`Unknown path in map ${current.toString()}.`);
    }
    getWithOrigin(current, typeIntrinsics) {
        var _a;
        const isa = this.getIsa();
        if (current !== null) {
            if (this.value.has(current)) {
                const sub = this.value.get(current);
                return {
                    value: sub,
                    origin: this
                };
            }
            else if (isa === null || isa === void 0 ? void 0 : isa.has(current)) {
                return isa.getWithOrigin(current, typeIntrinsics);
            }
            const intrinsics = (_a = typeIntrinsics.map) !== null && _a !== void 0 ? _a : CustomMap.getIntrinsics();
            if (intrinsics.has(current)) {
                return {
                    value: intrinsics.get(current),
                    origin: null
                };
            }
        }
        throw new Error(`Unknown path in map ${current.toString()}.`);
    }
    createInstance() {
        const newInstance = new CustomMap(new object_value_1.ObjectValue());
        newInstance.value.set(string_1.Isa, this);
        newInstance.isInstance = true;
        return newInstance;
    }
    getIsa() {
        const isa = this.value.get(string_1.Isa);
        return isa instanceof CustomMap ? isa : null;
    }
    hash(recursionDepth = 0) {
        let result = (0, hash_1.getHashCode)(this.value.size);
        if (recursionDepth > 4)
            return result;
        this.value.forEach((value, key) => {
            result ^= key.hash(recursionDepth + 1);
            result ^= value.hash(recursionDepth + 1);
        });
        return result;
    }
}
CustomMap.intrinsics = new object_value_1.ObjectValue();
exports.CustomMap = CustomMap;
