"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Zero = exports.PositiveOne = exports.NegativeOne = exports.CustomNumber = exports.CustomNumberIterator = void 0;
const hash_1 = require("../utils/hash");
const object_value_1 = require("../utils/object-value");
const with_intrinsics_1 = require("./with-intrinsics");
class CustomNumberIterator {
    constructor() {
        this.index = 0;
    }
    next() {
        return {
            value: null,
            done: true
        };
    }
}
exports.CustomNumberIterator = CustomNumberIterator;
class CustomNumber extends with_intrinsics_1.CustomValueWithIntrinsics {
    constructor(value) {
        super();
        this.id = `n:${value.toString()}`;
        this.value = value;
    }
    getCustomType() {
        return 'number';
    }
    toJSON() {
        return this.value.toString();
    }
    toString() {
        return this.value.toString();
    }
    fork() {
        return new CustomNumber(this.value);
    }
    toInt() {
        return this.value | 0;
    }
    toNumber() {
        return this.value;
    }
    toTruthy() {
        return !!this.value;
    }
    instanceOf(v, typeIntrinsics) {
        var _a;
        return v.value === ((_a = typeIntrinsics.number) !== null && _a !== void 0 ? _a : CustomNumber.intrinsics);
    }
    [Symbol.iterator]() {
        return new CustomNumberIterator();
    }
    has(_path) {
        return false;
    }
    set(_path, _newValue) {
        throw new Error('Mutable operations are not allowed on a number.');
    }
    get(current, typeIntrinsics) {
        var _a;
        const intrinsics = (_a = typeIntrinsics.number) !== null && _a !== void 0 ? _a : CustomNumber.getIntrinsics();
        if (intrinsics.has(current)) {
            return intrinsics.get(current);
        }
        throw new Error(`Unknown path in number ${current.toString()}.`);
    }
    getWithOrigin(current, typeIntrinsics) {
        return {
            value: this.get(current, typeIntrinsics),
            origin: null
        };
    }
    hash() {
        return (0, hash_1.getHashCode)(this.value);
    }
}
CustomNumber.intrinsics = new object_value_1.ObjectValue();
exports.CustomNumber = CustomNumber;
exports.NegativeOne = new CustomNumber(-1);
exports.PositiveOne = new CustomNumber(1);
exports.Zero = new CustomNumber(0);
