"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Outer = exports.Locals = exports.Globals = exports.Super = exports.Isa = exports.Self = exports.CustomString = exports.CustomStringIterator = void 0;
const keywords_1 = require("../byte-compiler/keywords");
const hash_1 = require("../utils/hash");
const object_value_1 = require("../utils/object-value");
const number_1 = require("./number");
const with_intrinsics_1 = require("./with-intrinsics");
class CustomStringIterator {
    constructor(value) {
        const me = this;
        me.value = value;
        me.index = 0;
    }
    next() {
        const me = this;
        if (me.index >= me.value.length) {
            return {
                value: null,
                done: true
            };
        }
        return {
            value: new CustomString(me.value[me.index++]),
            done: false
        };
    }
}
exports.CustomStringIterator = CustomStringIterator;
class CustomString extends with_intrinsics_1.CustomValueWithIntrinsics {
    static getCharIndex(item, index) {
        let n = index | 0;
        if (n < 0)
            n += item.value.length;
        if (n < 0 || n >= item.value.length)
            return -1;
        return n;
    }
    constructor(value) {
        super();
        this.id = `s:${value}`;
        this.value = value;
    }
    getCustomType() {
        return 'string';
    }
    toJSON() {
        return `"${this.value}"`;
    }
    toString() {
        return this.value;
    }
    fork() {
        return new CustomString(this.value);
    }
    isNumber() {
        const nr = Number(this.value);
        return !Number.isNaN(nr);
    }
    parseFloat() {
        return parseFloat(this.value);
    }
    parseInt() {
        return parseInt(this.value);
    }
    toNumber() {
        return 0;
    }
    toInt() {
        return 0;
    }
    toTruthy() {
        return this.value.length > 0;
    }
    instanceOf(v, typeIntrinsics) {
        var _a;
        return v.value === ((_a = typeIntrinsics.string) !== null && _a !== void 0 ? _a : CustomString.intrinsics);
    }
    slice(a, b) {
        return new CustomString(this.value.slice(a.toNumber(), b.toNumber()));
    }
    [Symbol.iterator]() {
        return new CustomStringIterator(this.value);
    }
    getCharIndex(index) {
        return CustomString.getCharIndex(this, index);
    }
    has(current) {
        if (current instanceof number_1.CustomNumber) {
            const index = current.toInt();
            return !!this.value[index];
        }
        return false;
    }
    set(_path, _newValue) {
        throw new Error('Mutable operations are not allowed on a string.');
    }
    get(current, typeIntrinsics) {
        var _a;
        if (current instanceof number_1.CustomNumber) {
            const currentIndex = this.getCharIndex(current.toInt());
            const segment = this.value[currentIndex];
            if (segment) {
                return new CustomString(segment);
            }
            throw new Error(`Index error (string index ${currentIndex} out of range).`);
        }
        const intrinsics = (_a = typeIntrinsics.string) !== null && _a !== void 0 ? _a : CustomString.getIntrinsics();
        if (intrinsics.has(current)) {
            return intrinsics.get(current);
        }
        throw new Error(`Unknown path in string ${current.toString()}.`);
    }
    getWithOrigin(current, typeIntrinsics) {
        return {
            value: this.get(current, typeIntrinsics),
            origin: null
        };
    }
    hash() {
        return (0, hash_1.getStringHashCode)(this.value);
    }
}
CustomString.intrinsics = new object_value_1.ObjectValue();
exports.CustomString = CustomString;
exports.Self = new CustomString(keywords_1.RuntimeKeyword.Self);
exports.Isa = new CustomString(keywords_1.RuntimeKeyword.Isa);
exports.Super = new CustomString(keywords_1.RuntimeKeyword.Super);
exports.Globals = new CustomString(keywords_1.RuntimeKeyword.Globals);
exports.Locals = new CustomString(keywords_1.RuntimeKeyword.Locals);
exports.Outer = new CustomString(keywords_1.RuntimeKeyword.Outer);
