"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.deepEqual = void 0;
const with_intrinsics_1 = require("../types/with-intrinsics");
function deepEqual(a, b) {
    const stack = [];
    const visited = new Set();
    stack.push([a, b]);
    while (stack.length > 0) {
        const [a, b] = stack.pop();
        visited.add(`${a.id}:${b.id}`);
        if (a instanceof with_intrinsics_1.CustomObject) {
            if (!(b instanceof with_intrinsics_1.CustomObject))
                return false;
            if (Array.isArray(a.value)) {
                if (!Array.isArray(b.value))
                    return false;
                if (a.value.length !== b.value.length)
                    return false;
                for (let index = 0; index < a.value.length; index++) {
                    const valueA = a.value[index];
                    const valueB = b.value[index];
                    if (!visited.has(`${valueA.id}:${valueB.id}`)) {
                        stack.push([valueA, valueB]);
                    }
                }
            }
            else {
                if (a.value.size !== b.value.size)
                    return false;
                for (const key of a.value.keys()) {
                    if (!b.value.has(key))
                        return false;
                    const valueA = a.value.get(key);
                    const valueB = b.value.get(key);
                    if (!visited.has(`${valueA.id}:${valueB.id}`)) {
                        stack.push([valueA, valueB]);
                    }
                }
            }
        }
        else if (a.value !== b.value) {
            return false;
        }
    }
    return true;
}
exports.deepEqual = deepEqual;
