"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.deepHash = void 0;
const with_intrinsics_1 = require("../types/with-intrinsics");
const hash_1 = require("./hash");
function deepHash(value) {
    let result = 0;
    const stack = [];
    const visited = new Set();
    stack.push([value]);
    while (stack.length > 0) {
        const items = stack.pop();
        for (const item of items) {
            visited.add(item.id);
            if (item instanceof with_intrinsics_1.CustomObject) {
                if (Array.isArray(item.value)) {
                    result ^= (0, hash_1.getHashCode)(item.value.length);
                    for (let index = item.value.length - 1; index >= 0; index--) {
                        const child = item.value[index];
                        if (!(child instanceof with_intrinsics_1.CustomObject) || !visited.has(child.id))
                            stack.push([child]);
                    }
                }
                else {
                    result ^= (0, hash_1.getHashCode)(item.value.length);
                    const chunk = [];
                    item.value.forEach((child, key) => {
                        if (!(key instanceof with_intrinsics_1.CustomObject) || !visited.has(key.id))
                            chunk.push(key);
                        if (!(child instanceof with_intrinsics_1.CustomObject) || !visited.has(child.id))
                            chunk.push(child);
                    });
                    stack.push(chunk);
                }
            }
            else {
                result ^= item.hash();
            }
        }
        result ^= (0, hash_1.rotateBits)(result);
    }
    return result;
}
exports.deepHash = deepHash;
