"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getStringHashCode = exports.getHashCode = exports.rotateBits = void 0;
const lru_cache_1 = require("lru-cache");
function rotateBits(n) {
    return (n >> 1) | (n << 31);
}
exports.rotateBits = rotateBits;
function getHashCode(value, offset = 0) {
    let unsigned = value >>> 0;
    unsigned = ((unsigned >> 16) ^ unsigned) * 0x45d9f3b;
    unsigned = ((unsigned >> 16) ^ unsigned) * 0x45d9f3b;
    unsigned = (unsigned >> 16) ^ unsigned;
    return ((offset << 5) - offset + unsigned) | 0;
}
exports.getHashCode = getHashCode;
exports.getStringHashCode = (function () {
    const cache = new lru_cache_1.LRUCache({
        ttl: 1000 * 60 * 5,
        max: 500
    });
    const generateHash = (value) => {
        let hash = 0;
        for (let i = 0; i < value.length; i++) {
            const chr = value.charCodeAt(i);
            hash = getHashCode(chr, hash);
        }
        return hash;
    };
    return (value) => {
        if (value.length === 0) {
            return 0;
        }
        if (cache.has(value)) {
            return cache.get(value);
        }
        const hash = generateHash(value);
        cache.set(value, hash);
        return hash;
    };
})();
