/// <reference types="node" />
import { OperationContext } from "./context";
import { ContextTypeIntrinsics } from "./context/types";
import { HandlerContainer } from "./handler-container";
import { CustomValue } from "./types/base";
import { Instruction, SourceLocation } from "./byte-compiler/instruction";
import { Stack } from "./utils/stack";
import EventEmitter from "events";
export declare class Debugger {
    private breakpoint;
    private nextStep;
    private vm;
    debug(...segments: any[]): CustomValue;
    setBreakpoint(breakpoint: boolean): Debugger;
    getBreakpoint(vm: VM): boolean;
    next(): Debugger;
    resume(): Promise<void>;
    interact(vm: VM, _source: SourceLocation): void;
}
export interface FrameOptions {
    code: Instruction[];
    self?: CustomValue;
    super?: CustomValue;
    outer?: OperationContext;
}
export declare enum VMState {
    PREPARATION = 0,
    PENDING = 1,
    FINISHED = 2,
    STOPPED = 3
}
export interface VMOptions {
    target: string;
    contextTypeIntrinsics: ContextTypeIntrinsics;
    globals: OperationContext;
    handler: HandlerContainer;
    debugger: Debugger;
    environmentVariables?: Map<string, string>;
    imports?: Map<string, Instruction[]>;
    externalFrames?: Stack<OperationContext>;
    maxActionsPerLoop?: number;
}
export declare class VM {
    private readonly ACTIONS_PER_LOOP;
    private readonly MAX_FRAMES;
    private maxActionsPerLoop;
    private actionCount;
    private state;
    private frames;
    private signal;
    private readonly STACK_LIMIT;
    private sp;
    private stack;
    private time;
    readonly target: string;
    readonly contextTypeIntrinsics: ContextTypeIntrinsics;
    readonly environmentVariables: Map<string, string>;
    readonly handler: HandlerContainer;
    readonly debugger: Debugger;
    readonly imports: Map<string, Instruction[]>;
    readonly externalFrames: Stack<OperationContext>;
    constructor(options: VMOptions);
    getTime(): number;
    isPending(): boolean;
    getStacktrace(): Instruction[];
    setMaxActionsPerLoop(actions: number): this;
    getOpenHandles(): number;
    getSignal(): EventEmitter;
    exit(): void;
    getFrame(): OperationContext;
    getFrames(): Stack<OperationContext>;
    private pushStack;
    private popStack;
    private createFrame;
    private popFrame;
    exec(): Promise<void>;
    private resume;
}
