"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.callWithContext = exports.call = void 0;
const keywords_1 = require("../byte-compiler/keywords");
const map_1 = require("../types/map");
function call(frame, fn, args) {
    var _a;
    const argsCount = args.length;
    for (let index = 0; index < argsCount; index++) {
        const argument = args.shift();
        const paramNum = argsCount - 1 - index;
        if (paramNum >= fn.arguments.length) {
            throw new Error('Too many arguments.');
        }
        const param = fn.arguments[paramNum].name;
        if (param.toString() === keywords_1.RuntimeKeyword.Self) {
            frame.self = argument;
            frame.super =
                argument instanceof map_1.CustomMap
                    ? (_a = argument.getIsa()) !== null && _a !== void 0 ? _a : new map_1.CustomMap()
                    : null;
        }
        else {
            frame.set(param, argument);
        }
    }
    for (let paramNum = argsCount; paramNum < fn.arguments.length; paramNum++) {
        frame.set(fn.arguments[paramNum].name, fn.arguments[paramNum].defaultValue);
    }
    frame.injectContext();
}
exports.call = call;
function callWithContext(frame, fn, args) {
    const argsCount = args.length;
    const selfParam = fn.arguments.length > 0 &&
        fn.arguments[0].name.toString() === keywords_1.RuntimeKeyword.Self
        ? 1
        : 0;
    for (let index = 0; index < argsCount; index++) {
        const argument = args.shift();
        const paramNum = argsCount - 1 - index + selfParam;
        if (paramNum >= fn.arguments.length) {
            throw new Error('Too many arguments.');
        }
        const param = fn.arguments[paramNum].name;
        if (param.toString() !== keywords_1.RuntimeKeyword.Self)
            frame.set(param, argument);
    }
    for (let paramNum = argsCount + selfParam; paramNum < fn.arguments.length; paramNum++) {
        frame.set(fn.arguments[paramNum].name, fn.arguments[paramNum].defaultValue);
    }
    frame.injectContext();
}
exports.callWithContext = callWithContext;
