"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.evalOr = exports.evalAnd = exports.evalPow = exports.evalMod = exports.evalNotEqual = exports.evalEqual = exports.evalGreaterThanOrEqual = exports.evalGreaterThan = exports.evalLessThanOrEqual = exports.evalLessThan = exports.evalBitwiseLeftShift = exports.evalBitwiseUnsignedRightShift = exports.evalBitwiseRightShift = exports.evalBitwiseAnd = exports.evalBitwiseOr = exports.evalDiv = exports.evalMul = exports.evalSub = exports.evalAdd = exports.absClamp01 = void 0;
const boolean_1 = require("../types/boolean");
const default_1 = require("../types/default");
const function_1 = require("../types/function");
const list_1 = require("../types/list");
const map_1 = require("../types/map");
const nil_1 = require("../types/nil");
const number_1 = require("../types/number");
const string_1 = require("../types/string");
const deep_equal_1 = require("../utils/deep-equal");
const absClamp01 = (d) => {
    if (d < 0)
        d = -d;
    if (d > 1)
        return 1;
    return d;
};
exports.absClamp01 = absClamp01;
function evalAdd(a, b) {
    if (a instanceof string_1.CustomString || b instanceof string_1.CustomString) {
        if (a instanceof nil_1.CustomNil)
            a = new string_1.CustomString('');
        if (b instanceof nil_1.CustomNil)
            b = new string_1.CustomString('');
        return new string_1.CustomString(a.toString() + b.toString());
    }
    else if (a instanceof number_1.CustomNumber) {
        if (b instanceof nil_1.CustomNil)
            b = new number_1.CustomNumber(0);
        if (b instanceof number_1.CustomNumber)
            return new number_1.CustomNumber(a.toNumber() + b.toNumber());
    }
    else if (a instanceof list_1.CustomList) {
        if (b instanceof list_1.CustomList) {
            return a.fork().extend(b);
        }
        return default_1.DefaultType.Void;
    }
    else if (a instanceof map_1.CustomMap) {
        if (b instanceof map_1.CustomMap) {
            return a.fork().extend(b);
        }
        return default_1.DefaultType.Void;
    }
    return default_1.DefaultType.Void;
}
exports.evalAdd = evalAdd;
const minusString = (a, b) => {
    const origin = a.toString();
    const toRemove = b.toString();
    if (origin.endsWith(toRemove)) {
        return new string_1.CustomString(origin.substring(0, origin.length - toRemove.length));
    }
    return new string_1.CustomString(origin);
};
function evalSub(a, b) {
    if (a instanceof string_1.CustomString) {
        if (b instanceof nil_1.CustomNil)
            b = new string_1.CustomString('');
        return minusString(a, b);
    }
    else if (a instanceof number_1.CustomNumber) {
        if (b instanceof nil_1.CustomNil)
            b = new number_1.CustomNumber(0);
        if (b instanceof number_1.CustomNumber)
            return new number_1.CustomNumber(a.toNumber() - b.toNumber());
    }
    return default_1.DefaultType.Void;
}
exports.evalSub = evalSub;
const multiplyString = (a, b) => {
    const factor = b.toNumber();
    if (factor <= 0) {
        return new string_1.CustomString('');
    }
    let newString = '';
    const max = Math.floor(a.value.length * factor);
    for (let index = 0; index < max; index++) {
        newString += a.value[index % a.value.length];
    }
    return new string_1.CustomString(newString);
};
const multiplyList = (a, b) => {
    const factor = b.toNumber();
    if (factor <= 0) {
        return new list_1.CustomList();
    }
    const newListValue = [];
    const max = Math.floor(a.value.length * factor);
    for (let index = 0; index < max; index++) {
        newListValue.push(a.value[index % a.value.length]);
    }
    return new list_1.CustomList(newListValue);
};
function evalMul(a, b) {
    if (a instanceof string_1.CustomString) {
        if (b instanceof nil_1.CustomNil)
            b = new string_1.CustomString('');
        return multiplyString(a, b);
    }
    else if (a instanceof number_1.CustomNumber) {
        if (b instanceof nil_1.CustomNil)
            b = new number_1.CustomNumber(0);
        if (b instanceof number_1.CustomNumber)
            return new number_1.CustomNumber(a.toNumber() * b.toNumber());
    }
    else if (a instanceof list_1.CustomList && b instanceof number_1.CustomNumber) {
        return multiplyList(a, b);
    }
    return default_1.DefaultType.Void;
}
exports.evalMul = evalMul;
const divideString = (a, b) => {
    const factor = 1 / b.toNumber();
    if (factor <= 0) {
        return new string_1.CustomString('');
    }
    let newString = '';
    const max = Math.floor(a.value.length * factor);
    for (let index = 0; index < max; index++) {
        newString += a.value[index % a.value.length];
    }
    return new string_1.CustomString(newString);
};
const divideList = (a, b) => {
    const factor = 1 / b.toNumber();
    if (factor <= 0) {
        return new list_1.CustomList();
    }
    const newListValue = [];
    const max = Math.floor(a.value.length * factor);
    for (let index = 0; index < max; index++) {
        newListValue.push(a.value[index % a.value.length]);
    }
    return new list_1.CustomList(newListValue);
};
function evalDiv(a, b) {
    if (a instanceof string_1.CustomString) {
        if (b instanceof nil_1.CustomNil)
            b = new string_1.CustomString('');
        return divideString(a, b);
    }
    else if (a instanceof number_1.CustomNumber) {
        if (b instanceof nil_1.CustomNil)
            b = new number_1.CustomNumber(0);
        if (b instanceof number_1.CustomNumber)
            return new number_1.CustomNumber(a.toNumber() / b.toNumber());
    }
    else if (a instanceof list_1.CustomList && b instanceof number_1.CustomNumber) {
        return divideList(a, b);
    }
    return default_1.DefaultType.Void;
}
exports.evalDiv = evalDiv;
function evalBitwiseOr(a, b) {
    if (a instanceof number_1.CustomNumber) {
        if (b instanceof nil_1.CustomNil)
            b = new number_1.CustomNumber(0);
        if (b instanceof number_1.CustomNumber)
            return new number_1.CustomNumber(a.toNumber() | b.toNumber());
    }
    return default_1.DefaultType.Void;
}
exports.evalBitwiseOr = evalBitwiseOr;
function evalBitwiseAnd(a, b) {
    if (a instanceof number_1.CustomNumber) {
        if (b instanceof nil_1.CustomNil)
            b = new number_1.CustomNumber(0);
        if (b instanceof number_1.CustomNumber)
            return new number_1.CustomNumber(a.toNumber() & b.toNumber());
    }
    return default_1.DefaultType.Void;
}
exports.evalBitwiseAnd = evalBitwiseAnd;
function evalBitwiseRightShift(a, b) {
    if (a instanceof number_1.CustomNumber) {
        if (b instanceof nil_1.CustomNil)
            b = new number_1.CustomNumber(0);
        if (b instanceof number_1.CustomNumber)
            return new number_1.CustomNumber(a.toNumber() >> b.toNumber());
    }
    return default_1.DefaultType.Void;
}
exports.evalBitwiseRightShift = evalBitwiseRightShift;
function evalBitwiseUnsignedRightShift(a, b) {
    if (a instanceof number_1.CustomNumber) {
        if (b instanceof nil_1.CustomNil)
            b = new number_1.CustomNumber(0);
        if (b instanceof number_1.CustomNumber)
            return new number_1.CustomNumber(a.toNumber() >>> b.toNumber());
    }
    return default_1.DefaultType.Void;
}
exports.evalBitwiseUnsignedRightShift = evalBitwiseUnsignedRightShift;
function evalBitwiseLeftShift(a, b) {
    if (a instanceof number_1.CustomNumber) {
        if (b instanceof nil_1.CustomNil)
            b = new number_1.CustomNumber(0);
        if (b instanceof number_1.CustomNumber)
            return new number_1.CustomNumber(a.toNumber() << b.toNumber());
    }
    return default_1.DefaultType.Void;
}
exports.evalBitwiseLeftShift = evalBitwiseLeftShift;
function evalLessThan(a, b) {
    if (a instanceof number_1.CustomNumber) {
        if (b instanceof nil_1.CustomNil)
            b = new number_1.CustomNumber(0);
        if (b instanceof number_1.CustomNumber)
            return new boolean_1.CustomBoolean(a.toNumber() < b.toNumber());
    }
    else if (a instanceof string_1.CustomString) {
        if (b instanceof nil_1.CustomNil)
            b = new string_1.CustomString('');
        return new boolean_1.CustomBoolean(a.toString() < b.toString());
    }
    return default_1.DefaultType.Void;
}
exports.evalLessThan = evalLessThan;
function evalLessThanOrEqual(a, b) {
    if (a instanceof number_1.CustomNumber) {
        if (b instanceof nil_1.CustomNil)
            b = new number_1.CustomNumber(0);
        if (b instanceof number_1.CustomNumber)
            return new boolean_1.CustomBoolean(a.toNumber() <= b.toNumber());
    }
    else if (a instanceof string_1.CustomString) {
        if (b instanceof nil_1.CustomNil)
            b = new string_1.CustomString('');
        return new boolean_1.CustomBoolean(a.toString() <= b.toString());
    }
    return default_1.DefaultType.Void;
}
exports.evalLessThanOrEqual = evalLessThanOrEqual;
function evalGreaterThan(a, b) {
    if (a instanceof number_1.CustomNumber) {
        if (b instanceof nil_1.CustomNil)
            b = new number_1.CustomNumber(0);
        if (b instanceof number_1.CustomNumber)
            return new boolean_1.CustomBoolean(a.toNumber() > b.toNumber());
    }
    else if (a instanceof string_1.CustomString) {
        if (b instanceof nil_1.CustomNil)
            b = new string_1.CustomString('');
        return new boolean_1.CustomBoolean(a.toString() > b.toString());
    }
    return default_1.DefaultType.Void;
}
exports.evalGreaterThan = evalGreaterThan;
function evalGreaterThanOrEqual(a, b) {
    if (a instanceof number_1.CustomNumber) {
        if (b instanceof nil_1.CustomNil)
            b = new number_1.CustomNumber(0);
        if (b instanceof number_1.CustomNumber)
            return new boolean_1.CustomBoolean(a.toNumber() >= b.toNumber());
    }
    else if (a instanceof string_1.CustomString && b instanceof string_1.CustomString) {
        if (b instanceof nil_1.CustomNil)
            b = new string_1.CustomString('');
        return new boolean_1.CustomBoolean(a.toString() >= b.toString());
    }
    return default_1.DefaultType.Void;
}
exports.evalGreaterThanOrEqual = evalGreaterThanOrEqual;
function evalEqual(a, b) {
    if (a instanceof number_1.CustomNumber) {
        return new boolean_1.CustomBoolean(a.value === b.value);
    }
    else if (a instanceof string_1.CustomString) {
        return new boolean_1.CustomBoolean(a.value === b.value);
    }
    else if (a instanceof list_1.CustomList && b instanceof list_1.CustomList) {
        return new boolean_1.CustomBoolean((0, deep_equal_1.deepEqual)(a, b));
    }
    else if (a instanceof map_1.CustomMap && b instanceof map_1.CustomMap) {
        return new boolean_1.CustomBoolean((0, deep_equal_1.deepEqual)(a, b));
    }
    else if (a instanceof function_1.CustomFunction) {
        return new boolean_1.CustomBoolean(a === b);
    }
    else if (a instanceof nil_1.CustomNil) {
        return new boolean_1.CustomBoolean(b instanceof nil_1.CustomNil);
    }
    return default_1.DefaultType.False;
}
exports.evalEqual = evalEqual;
function evalNotEqual(a, b) {
    if (a instanceof number_1.CustomNumber) {
        return new boolean_1.CustomBoolean(a.value !== b.value);
    }
    else if (a instanceof string_1.CustomString) {
        return new boolean_1.CustomBoolean(a.value !== b.value);
    }
    else if (a instanceof list_1.CustomList && b instanceof list_1.CustomList) {
        return new boolean_1.CustomBoolean(!(0, deep_equal_1.deepEqual)(a, b));
    }
    else if (a instanceof map_1.CustomMap && b instanceof map_1.CustomMap) {
        return new boolean_1.CustomBoolean(!(0, deep_equal_1.deepEqual)(a, b));
    }
    else if (a instanceof function_1.CustomFunction) {
        return new boolean_1.CustomBoolean(a !== b);
    }
    else if (a instanceof nil_1.CustomNil) {
        return new boolean_1.CustomBoolean(!(b instanceof nil_1.CustomNil));
    }
    return default_1.DefaultType.True;
}
exports.evalNotEqual = evalNotEqual;
function evalMod(a, b) {
    if (a instanceof number_1.CustomNumber) {
        if (b instanceof nil_1.CustomNil)
            b = new number_1.CustomNumber(0);
        if (b instanceof number_1.CustomNumber)
            return new number_1.CustomNumber(a.toNumber() % b.toNumber());
    }
    return default_1.DefaultType.Void;
}
exports.evalMod = evalMod;
function evalPow(a, b) {
    if (a instanceof number_1.CustomNumber) {
        if (b instanceof nil_1.CustomNil)
            b = new number_1.CustomNumber(0);
        if (b instanceof number_1.CustomNumber)
            return new number_1.CustomNumber(Math.pow(a.toNumber(), b.toNumber()));
    }
    return default_1.DefaultType.Void;
}
exports.evalPow = evalPow;
function evalAnd(a, b) {
    const left = Number(a instanceof number_1.CustomNumber ? a.toNumber() : a.toTruthy());
    const right = Number(b instanceof number_1.CustomNumber ? b.toNumber() : b.toTruthy());
    return new number_1.CustomNumber((0, exports.absClamp01)(left * right));
}
exports.evalAnd = evalAnd;
function evalOr(a, b) {
    const left = Number(a instanceof number_1.CustomNumber ? a.toNumber() : a.toTruthy());
    const right = Number(b instanceof number_1.CustomNumber ? b.toNumber() : b.toTruthy());
    return new number_1.CustomNumber((0, exports.absClamp01)(left + right - left * right));
}
exports.evalOr = evalOr;
